Package: MassArray
Type: Package
Title: Analytical Tools for MassArray Data
Version: 1.62.0
Date: 2019-03-18
Author: Reid F. Thompson <reid.thompson@gmail.com>, John M. Greally <john.greally@einstein.yu.edu>
Maintainer: Reid F. Thompson <reid.thompson@gmail.com>
Imports: graphics, grDevices, stats, utils
Depends: R (>= 2.10.0), methods
Description: This package is designed for the import, quality control, analysis, and visualization of methylation data generated using Sequenom's MassArray platform.  The tools herein contain a highly detailed amplicon prediction for optimal assay design.  Also included are quality control measures of data, such as primer dimer and bisulfite conversion efficiency estimation.  Methylation data are calculated using the same algorithms contained in the EpiTyper software package.  Additionally, automatic SNP-detection can be used to flag potentially confounded data from specific CG sites.  Visualization includes barplots of methylation data as well as UCSC Genome Browser-compatible BED tracks.  Multiple assays can be positionally combined for integrated analysis.
License: GPL (>=2)
biocViews: ImmunoOncology, DNAMethylation, SNP, MassSpectrometry,
        Genetics, DataImport, Visualization
LazyData: yes
LazyLoad: yes
git_url: https://git.bioconductor.org/packages/MassArray
git_branch: RELEASE_3_22
git_last_commit: 2c04299
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 04:50:19 UTC; biocbuild
